Scriptname mf_SpectatorCrowdsGetSpectators extends Quest 
; The Spectator Pooler maintains and updates the list of valid spectators in the cell
; It has functions used to find valid spectators for a given sex target

SexLabFramework Property SexLab Auto

mf_SpectatorCrowdsConfig Property SCConfig Auto
mf_SpectatorCrowdsListPotentialSpect Property ListPotentialSpectators Auto
Quest Property FetchSpectators Auto

Actor[] PotentialSpectator

int Property FetchedSpectatorCount Auto

bool threadLock = false
bool Property spectatorsPopulated Auto

Function AddPotentialSpectator(Actor akPotentialSpectator)
	PotentialSpectator[FetchedSpectatorCount]=akPotentialSpectator
	FetchedSpectatorCount += 1
endFunction

; this function fetches the requested number of spectators around the target 
Actor[] Function GetSpectatorsNearestTarget(ObjectReference SexTarget, int RequestedCount)
	; we want to prevent another thread from using this function at the same time
	while threadLock
		Utility.wait(0.1)
	endWhile
	threadLock = true
	spectatorsPopulated = false
	PotentialSpectator = New Actor[128] ; contains references for the potential spectators
	FetchedSpectatorCount = 0
	;MyDebug("About to find spectators for SexTarget: " + (SexTarget As Actor).getLeveledActorBase().getName())
	FetchSpectators.Start() ; populates the PotentialSpectator array
	while (!spectatorsPopulated)
		Utility.wait(0.1)
	endWhile
	;MyDebug("Spectators found, calculating distances")
	
	Float[] PotentialSpectatorDistance = New float[128] ; contains distances for the potential spectators

	int i=0
	while (i < FetchedSpectatorCount)
		PotentialSpectatorDistance[i] = SexTarget.GetDistance(PotentialSpectator[i] as ObjectReference)
		i+= 1
	endWhile
	;MyDebug("Spectators chosen and distances calculated")
	SortArraysByDistance(PotentialSpectator, PotentialSpectatorDistance, FetchedSpectatorCount)
	;MyDebug("Spectators sorted")	
	Actor[] SelectedSpectators = New Actor[128] ; contains the spectators selected for this particular thread
	
	;MyDebug("FetchedSpectatorCount is " + FetchedSpectatorCount)

	float AggroDistance = SCConfig.AggroDistance.GetValue()
	i = 0
	int k = 0
	while (k < RequestedCount && i < FetchedSpectatorCount)
		if ((PotentialSpectatorDistance[i] < AggroDistance) || PotentialSpectator[i].HasLOS(SexTarget))
			;Debug.Trace("Adding Closest Spectator: " + PotentialSpectator[i].getLeveledActorBase().getName())
			SelectedSpectators[k] = PotentialSpectator[i]
			k+= 1
		endif
		i+= 1
	endWhile

	;MyDebug("Finished Fetching Spectators")
	threadLock = false

	return SelectedSpectators
		
EndFunction

Function SortArraysByDistance (Actor[] ActorArray, Float[] DistanceArray, int ArrayLength)
 
	Int Index1
	Int Index2 = ArrayLength - 1
 
	While (Index2 > 0)
		Index1 = 0
		While (Index1 < Index2)
			If (DistanceArray [Index1] > DistanceArray [Index1 + 1])
				Float SwapDummyDistance = DistanceArray [Index1]
				Actor SwapDummyActor = ActorArray [Index1]
				DistanceArray [Index1] = DistanceArray [Index1 + 1]
				ActorArray [Index1] = ActorArray [Index1 + 1]
				DistanceArray [Index1 + 1] = SwapDummyDistance
				ActorArray [Index1 + 1] = SwapDummyActor
			EndIf
			Index1 += 1
		EndWhile
		Index2 -= 1
	EndWhile
 
EndFunction

Function MyDebug(string mymessage)
	Debug.Trace(mymessage)
EndFunction

 
